@@ -11,14 +11,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Setup for authorization and secret management for Conduit.
+
+Utilizes Fernet for symmetric encryption of secrets used by Conduit.
+"""
 
 import os
 import logging
 from pathlib import Path
-from typing import Optional, Callable
+from typing import Any, Generator, Optional, Callable
 from contextlib import contextmanager
 import yaml
 from cryptography.fernet import Fernet
+from mypy_extensions import Arg, NamedArg
 
 logger = logging.getLogger(__name__)
 
@@ -27,8 +32,32 @@ secret_path: Optional[Path] = None
 
 secrets: Optional[dict] = None
 
+yaml_cfg_loader = Callable[[Arg(str, 'filename'),
+                            NamedArg(bool, 'no_exist_ok'),
+                            NamedArg(bool, 'local_only')],
+                            dict[str, str]]
 
-def setup(etc_dir: Path, load_cfg_from_yaml: Callable):
+
+def _require_setup(func: Callable[..., Any]) -> Any:
+    def wrapper(*args: Any, **kwargs: Any) -> Any:
+        if auth_dir is None or secret_path is None or secrets is None:
+            raise AuthNotSetupExeception
+
+        return func(*args, **kwargs)
+
+    return wrapper
+
+
+def setup(etc_dir: Path, load_cfg_from_yaml: yaml_cfg_loader) -> None:
+    """Sets up necessary directories for Conduit, and reads in secrets.
+
+    The `etc/auth/` and `etc/auth/secrets/` directores are created here.
+
+    Args:
+        etc_dir (Path): A `Path` to the etc directory.
+        load_cfg_from_yaml (Callable): Function that can load in a YAML config file, used
+            to read in the secrets file.
+    """
     logger.debug(f'etc_dir: {etc_dir}, load_cfg_from_yaml: {load_cfg_from_yaml}')
 
     # create paths
@@ -92,8 +121,23 @@ def setup(etc_dir: Path, load_cfg_from_yaml: Callable):
     logger.debug(f'loaded secrets: {list(secrets.keys())}')
 
 
+@_require_setup
 @contextmanager
-def get_secret(secret_id: str) -> str:
+def get_secret(secret_id: str) -> Generator[str, None, None]:
+    """Retrieves a secret temporarily.
+
+    Works as a context manager, where the secret will be deleted from memory upon
+    exiting.
+
+    Args:
+        secret_id (str): The ID of the secret to get.
+
+    Raises:
+        Exception: When no secret was associated with the `secret_id`.
+
+    Yields:
+        Iterator[str]: The decrypted secret.
+    """
     try:
         encrypted_password = secrets.get(secret_id, None)
 
@@ -112,7 +156,8 @@ def get_secret(secret_id: str) -> str:
         raise
 
 
-def _create_key_file():
+def _create_key_file() -> None:
+    """Creates a secret key file."""
     logger.info('Creating secret key')
     with secret_path.open('wb') as f:
         key = Fernet.generate_key()
@@ -122,25 +167,57 @@ def _create_key_file():
     os.chmod(secret_path, 0o400)
 
 
+@_require_setup
 def _encrypt(plaintext: str) -> bytes:
+    """Encrypts the given text.
+
+    Args:
+        plaintext (str): Text to encrypt.
+
+    Returns:
+        bytes: The encrypted text.
+    """
     with _get_fernet() as f:
         return f.encrypt(plaintext.encode())
 
 
+@_require_setup
 def _decrypt(ciphertext: bytes) -> str:
+    """Decrypt the given ciphertext.
+
+    Args:
+        ciphertext (bytes): Encrypted text to decrypt.
+
+    Returns:
+        str: The decrypted text.
+    """
     with _get_fernet() as f:
         return f.decrypt(ciphertext).decode()
 
 
+@_require_setup
 @contextmanager
-def _get_fernet() -> Fernet:
+def _get_fernet() -> Generator[Fernet, None, None]:
+    """Creates a `Fernet` to allow for encryption and decryption using the secret key.
+
+    Returns:
+        Fernet: Allows for encryption and decryption utilizing your secret key.
+
+    Yields:
+        Iterator[Fernet]: The initialized `Fernet`, which is deleted from memory after use.
+    """
     with secret_path.open() as f:
         key = f.read()
 
-    fern = Fernet(key)
+    fern = Fernet(key)  # type: ignore
 
     try:
         yield fern
     finally:
         del fern
         del key
+
+
+class AuthNotSetupExeception(Exception):
+    """Raised when authorization and secrets not setup yet."""
+    pass
